/* pam_data.c */

#include <stdlib.h>
#include <string.h>

#include "pam_appl.h"
#include "pam_private.h"

struct pam_data *_pam_locate_data(const pam_handle_t *pamh, const char *name);

int pam_set_data(
    pam_handle_t *pamh,
    const char *module_data_name,
    const void *data,
    int (*cleanup)(pam_handle_t *pamh, void *data, int error_status))
{
    struct pam_data *data_entry;

    if ((data_entry = _pam_locate_data(pamh, module_data_name))) {
	/* Should cleanup() really be called here?
	 * According to SonOS 5.5 man page, no.
	 * if (data_entry->cleanup) {
	 *   data_entry->cleanup(pamh, data_entry->data, PAM_SUCCESS);
	 * }
	 */
	if (data_entry->name) {
	    free(data_entry->name);
	}
    } else {
	data_entry = malloc(sizeof(*data_entry));
	data_entry->next = pamh->data;
	pamh->data = data_entry;
    }

    data_entry->name = strdup(module_data_name);
    data_entry->data = data;
    data_entry->cleanup = cleanup;

    return PAM_SUCCESS;
}

int pam_get_data(
    const pam_handle_t *pamh,
    const char *module_data_name,
    void **datap)
{
    struct pam_data *data;

    data = _pam_locate_data(pamh, module_data_name);
    if (data) {
	*datap = data->data;
	return PAM_SUCCESS;
    }

    return PAM_NO_MODULE_DATA;
}

struct pam_data *_pam_locate_data(const pam_handle_t *pamh, const char *name)
{
    struct pam_data *data = pamh->data;
    
    while (data) {
	if (!strcmp(data->name, name)) {
	    return data;
	}
	data = data->next;
    }

    return NULL;
}


void _pam_free_data(pam_handle_t *pamh, int status)
{
    struct pam_data *data = pamh->data;
    struct pam_data *last;

    while (data) {
	last = data;
	data = data->next;
	if (last->cleanup) {
	    last->cleanup(pamh, last->data, status);
	}
	free(last->name);
	free(last);
    }
}
