/* pam_dispatch.c - handles module function dispatch */

#include <stdlib.h>

#include "pam_appl.h"
#include "pam_private.h"

static int _pam_dispatch_aux(pam_handle_t *pamh,
			     int flags, struct handler *h);

int _pam_dispatch(pam_handle_t *pamh, int flags, int select)
{
    struct handler *h = NULL;
    int res;

    /* Load all modules, resolve all symbols */
    /* XXX - is this the right response? */
    if ((res = _pam_init_handlers(pamh)) != PAM_SUCCESS) {
	_pam_log_error("unable to dispatch function");
	return res;
    }

    switch (select) {
    case PAM_AUTHENTICATE:  h = pamh->handlers.authenticate; break;
    case PAM_SETCRED:       h = pamh->handlers.setcred; break;
    case PAM_ACCOUNT:       h = pamh->handlers.acct_mgmt; break;
    case PAM_OPEN_SESSION:  h = pamh->handlers.open_session; break;
    case PAM_CLOSE_SESSION: h = pamh->handlers.close_session; break;
    case PAM_CHAUTHTOK:     h = pamh->handlers.chauthtok; break;
    }

    return _pam_dispatch_aux(pamh, flags, h);
}

static int _pam_dispatch_aux(
    pam_handle_t *pamh,
    int flags,
    struct handler *h)
{
    int ret;
    int result = PAM_SUCCESS;
    
    while (h) {
	ret = h->func(pamh, flags, h->argc, h->argv);
	if (ret == PAM_SUCCESS) {
	    if (h->control_flag == PAM_SUFFICIENT) {
		return PAM_SUCCESS;
	    }
	} else {
	    if (h->control_flag == PAM_REQUIRED) {
		result = ret;
	    }
	}

	h = h->next;
    }

    return result;
}
