/* pam_item.c */

#include <stdlib.h>
#include <string.h>

#include "pam_appl.h"
#include "pam_private.h"

#define RESET(X, Y) \
if (X) { \
     free(X); \
}; \
(X) = (Y) ? strdup(Y) : NULL

int pam_set_item (
    pam_handle_t *pamh,
    int item_type,
    const void *item)
{
    switch (item_type) {
    case PAM_SERVICE:
	/* Setting handlers_loaded to 0 will cause the handlers
	 * to be reloaded on the next call to a service module.
	 */
	pamh->handlers.handlers_loaded = 0;
	RESET(pamh->service_name, item);
	break;
    case PAM_USER:
	RESET(pamh->user, item);
	break;
    case PAM_TTY:
	RESET(pamh->tty, item);
	break;
    case PAM_RUSER:
	RESET(pamh->ruser, item);
	break;
    case PAM_RHOST:
	RESET(pamh->rhost, item);
	break;
    case PAM_AUTHTOK:
	/* XXX - The man page says this is only supposed to
	 * be available to the module providers.  Not sure
	 * how to do that...
	 */
	RESET(pamh->authtok, item);
	break;
    case PAM_OLDAUTHTOK:
	/* XXX - This too. */
	RESET(pamh->oldauthtok, item);
	break;
    case PAM_CONV:
	pamh->pam_conversation = item;
        break;
    default:
	return PAM_BAD_ITEM;
    }

    return PAM_SUCCESS;
}

int pam_get_item (
    const pam_handle_t *pamh,
    int item_type,
    void **item)
{
    switch (item_type) {
    case PAM_SERVICE:
	*item = pamh->service_name;
	break;
    case PAM_USER:
	*item = pamh->user;
	break;
    case PAM_TTY:
	*item = pamh->tty;
	break;
    case PAM_RUSER:
	*item = pamh->ruser;
	break;
    case PAM_RHOST:
	*item = pamh->rhost;
	break;
    case PAM_AUTHTOK:
	*item = pamh->authtok;
	break;
    case PAM_OLDAUTHTOK:
	*item = pamh->oldauthtok;
	break;
    case PAM_CONV:
	*item = pamh->pam_conversation;
	break;
    default:
	/* XXX - I made this up */
	return PAM_BAD_ITEM;
    }
  
    return PAM_SUCCESS;
}
