/*
 * pam_modules.h
 * 
 * This header file documents the PAM SPI --- that is, interface
 * between the PAM library and a PAM service library which is called
 * by the PAM library.
 *
 * Last modified: 15-Jan-96 by TYT
 */

/*
 * Copyright Theodore Ts'o, 1996.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, and the entire permission notice in its entirety,
 *    including the disclaimer of warranties.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 * 
 * ALTERNATIVELY, this product may be distributed under the terms of
 * the GNU Public License, in which case the provisions of the GPL are
 * required INSTEAD OF the above restrictions.  (This clause is
 * necessary due to a potential bad interaction between the GPL and
 * the restrictions contained in a BSD-style copyright.)
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _SECURITY_PAM_MODULES_H
#define _SECURITY_PAM_MODULES_H

#ifdef PAM_STATIC_LIBRARY
#define PAM_EXTERN static

struct pam_module {
	char *name;		/* Name of the module */
	/*
	 * These are function pointers to the module's key functions.
	 */
	int (*pam_sm_authenticate)(pam_handle_t *pamh, int flags,
				   int argc, const char **argv);
	int (*pam_sm_setcred)(pam_handle_t *pamh, int flags,
			      int argc, const char **argv);
	int (*pam_sm_acct_mgmt)(pam_handle_t *pamh, int flags,
				int argc, const char **argv);
	int (*pam_sm_open_session)(pam_handle_t *pamh, int flags,
				   int argc, const char **argv);
	int (*pam_sm_close_session)(pam_handle_t *pamh, int flags,
				    int argc, const char **argv);
	int (*pam_sm_chauthtok)(pam_handle_t *pamh, int flags,
				int argc, const char **argv);
	/*
	 * linked-list pointers
	 */
	struct pam_modules *prev, *next;
}
#else /* !PAM_STATIC_LIBRARY */
#define PAM_EXTERN extern
#endif /* PAM_STATIC LIBRARY */
	
/*
 * Authentication API's
 */
PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags,
				   int argc, const char **argv);
PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags,
			      int argc, const char **argv);

/*
 * Account Management API's
 */
PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
				int argc, const char **argv);

/*
 * Session Management API's
 */
PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
				   int argc, const char **argv);

PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags,
				    int argc, const char **argv);

/*
 * Password Management API's
 */
PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
				int argc, const char **argv);

#undef PAM_EXTERN


#endif /* _SECURITY_PAM_MODULES_H */
